/*
 * Name: Telephone number formatter
 * Developer: J.P. Fortier
 * Date: 5/10/2018

 * This library loads International Telephone Input v12.1.13
 * in both edit and view state. Using several parameters,
 * you can edit telephone numbers and specify national formats.

 * International Telephone Input v12.1.13
 * https://github.com/jackocnr/intl-tel-input.git
 * Licensed under the MIT license

 *  Copyright(C) 2010 The Libphonenumber Authors
 */
define(function () {
    // gets the option collection given to the library
    var getOptions = function (context) {
        var initialCountryParam = context.getParameter("InitialCountry");
        let preferredCountriesParam = context.getParameter("PreferredCountries");
        var onlyCountriesParam = context.getParameter("OnlyCountries");
        var allowDropdownParam = context.getParameter("AllowDropdown");
        var useNationalModeParam = context.getParameter("UseNationalMode");

        var options = {
            dropdownContainer: "body"
        }

        if (initialCountryParam) {
            options.initialCountry = initialCountryParam;
        }

        if (preferredCountriesParam) {
            options.preferredCountries = preferredCountriesParam.split(",");
        }

        if (onlyCountriesParam) {
            options.onlyCountries = onlyCountriesParam.split(",");
            options.allowDropdown = options.onlyCountries !== 1;
        }

        if (allowDropdownParam === "false") {
            options.allowDropdown = false;
        }

        if (useNationalModeParam === "false") {
            options.nationalMode = false;
        }

        return options;
    };

    var install = function (holderElement, context) {
        var telLibrary = context.loadedDependencies[0]; // null
        var utilsLibrary = context.loadedDependencies[1]; // null
        var $ = context.loadedDependencies[2];

        var $holderElement = $(holderElement.childNodes[0]);
        var cell = context.getCell();
        const fieldViewModel = cell.viewModel;

        var $inputElement = $holderElement.children().eq(0);
        var $viewElement = $holderElement.children().eq(1);

        // this configures the edit state version of this control
        var configureInput = function () {
            $viewElement.remove();

            $inputElement.val(cell.value);
            var options = getOptions(context);
            $inputElement.intlTelInput(options);

            if (options.allowDropdown === false) {
                $inputElement.prev().remove();
            }

            $inputElement.on("input.vinyl-telephone, countrychange.vinyl-telephone", function (event) {
                // new input
                fieldViewModel.touch();
            });

            const blurHandler = function (event) {
                if ($.contains(holderElement, event.relatedTarget)) {
                    // blurring to the country, ignore this
                    return;
                }

                // new input
                var intlNumber = $inputElement.intlTelInput("getNumber");

                if (intlNumber == cell.value) {
                    // no change
                    fieldViewModel.blur();
                    return;
                }

                cell.value = intlNumber;
            };
            $inputElement.on("blur.vinyl-telephone", blurHandler);

            // value changes
            // NOTE: Does not get removed when you switch to and from edit
            cell.setChangeCallback('value', function (value) {
                // input no longer in the DOM
                if (!$.contains(document.documentElement, $inputElement.get(0))) {
                    return;
                }

                // if the value changes, set it
                if (fieldViewModel.supportsAlwaysEditableMode() && fieldViewModel.mode() === 1) {
                    return;
                }

                if (value === null) {
                    $inputElement.val("");
                    return;
                }

                $inputElement.intlTelInput("setNumber", value);
            });
        }

        // this configures the view state version of the control.
        var configureViewState = function () {
			$inputElement.remove();

            var $textHolder = $viewElement.find(".telephone-input-view-value");
            var useNationalModeParam = context.getParameter("UseNationalMode");

            const setValue = function(phoneValue) {
				var format = useNationalModeParam === "false" ? window.intlTelInputUtils.numberFormat.INTERNATIONAL :
				                                                window.intlTelInputUtils.numberFormat.NATIONAL;
				            var number = window.intlTelInputUtils.formatNumber(phoneValue, null, format);
				            // get the value from the input
            	$textHolder.text(number);
			};
			setValue(cell.value);

            // value changes
			// NOTE: Does not get removed right away when you switch to and from edit
			cell.setChangeCallback('value', function (newValue) {
				setValue(newValue);
			});

            if (fieldViewModel.supportsAlwaysEditableMode()) {
                $viewElement.addClass("vinyl-control-always-edit");

                const switchToEdit = function () {
                    // run switch to edit state
                    if (!fieldViewModel.trySwitchFieldToEditState(false)) {
                        // we didn't actually switch to edit state, don't mess
                        // with field focus
                        return;
                    }

                    var subscription = fieldViewModel.rendered.subscribe(function (rendered) {
                        if (rendered) {
                            subscription.dispose();
                            $(holderElement).find("input").focus().select();
                        }
                    });
                }

                // add the switch to edit action to the click input and helper focus handler
                var $focusHelper = $viewElement.find(".telephone-input-view-tabhelper");
                $viewElement.on("click.telephone-alwaysedit", switchToEdit);
                $focusHelper.on("focus.telephone-alwaysedit", switchToEdit);
            }

            if (fieldViewModel.supportsAutoEdit()) {
                $viewElement.addClass("vinyl-control-autoedit");

                const switchRowToEdit = function () {
					$viewElement.parent().addClass('vinyl-auto-edit-source');
					fieldViewModel.switchParentToEditState();
				}

				// add the switch to edit action to the click input and helper focus handler
				$viewElement.on("click.telephone-autoedit", switchRowToEdit);
			}
        };

        // Today widgets are re-installed when you switch from view to edit.
        // So we don't need to worry about listening to edit state and doing the switch
        // ourselves.
        if (fieldViewModel.mode() === 1) { // Edit
            configureInput();
        } else {
            configureViewState();
        }

    };

    var uninstall = function (holderElement, context) {
        // Nothing to uninstall.
    };

    return {
        callbacks: {
            events: {
                // adding and removing
                install: install,
                uninstall: uninstall
            }
        }
    };
});